/*
  SDTest
  Read Data from an SD card, output it to pins at a specified rate 
*/
#include "simpletools.h"     // Include simple tools

const int DO = 22, CLK = 23, DI = 24, CS = 25;

#define BufferCount 4
#define TRUE 1
#define FALSE 0

char Buffer[BufferCount][512];
volatile char BufferFull[BufferCount];
volatile char StopSending = FALSE;

void OutputDataThread(void *par);

#define OutputStackSize (40 + 32)

static int OutputStack[OutputStackSize];

int val[1000];
int i;

int main(void)    // Main function
{
  high(3);
  sd_mount(DO, CLK, DI, CS);               // Mount SD card

//Generate the file data a clock for the ClearPath motor
  
  for(i=1; i<1000+1; i+=2)
  {
    val[i]=1;
    val[i+1]=0;
  }
  
  print("Start Program.\n");
  
//Write data to SD memory      

  FILE* fp = fopen("test.txt", "wb"); // Open a file for writing  
    
  for(i=1;i<1000+1;i++)
  {
    fwrite(&val[i], sizeof(val[i]),1,fp);
  }    
  fclose(fp);
                                 
//End writing data to the SD Memory 

  print("End of SD memory writing Program.\n"); 

//Start Jason's program 

  for(int i=0; i<BufferCount; i++)
  {
    BufferFull[i]=FALSE;
  }
  
  StopSending=FALSE;  
  cogstart(&OutputDataThread, 0, OutputStack, sizeof(OutputStack) );  
  sd_mount(DO, CLK, DI, CS);  
  FILE *fp1;  
  fp1 = fopen("test.txt", "rb"); 
  int bufferToRead = 0;  
  int readResult = 0;
  
  while(1)
  {
    while( BufferFull[bufferToRead] == TRUE )
    {
      ;
    }
    
    putChar('0' + bufferToRead);
    
    readResult = fread(Buffer[bufferToRead], 1, 512, fp1);
    
    if( readResult > 0)
    {
      if(readResult < 512 )
      {
        memset( Buffer[bufferToRead] + readResult, 0, 512 - readResult );
      }      
      BufferFull[ bufferToRead ] = TRUE;      
      bufferToRead = (bufferToRead + 1) % BufferCount;      
     }           
     else     
    {      
    break;      
    }
  }
  
  StopSending = TRUE;  
  fclose(fp1);
  
  while(TRUE)
  {
    ;
  }                   
}

void OutputDataThread(void *par)
{
  int OutputRate = 53000;  
  int OutputDelay = _clkfreq / OutputRate;  
  int bufferToSend = 0;  
  int PinMask = (1<<0) | ( 1<<1) | ( 1<<2) | (1<<3);  
  int DataShift = 0;  
  DIRA |= PinMask;  
  DIRA |= (1<<26);  
  int clockTick = CNT;
    
  while( StopSending == FALSE )
  {
    while( BufferFull[bufferToSend] == FALSE )
    {
      clockTick = CNT;
    }
        
    for( int i=0; i<512; i++ )
    {
      int data = Buffer[bufferToSend][i] & 0x0F;      
      clockTick += OutputDelay;     
      waitcnt(clockTick );      
      OUTA = (OUTA & ~PinMask) | (data << DataShift);
    }
    
    if( bufferToSend == 0 )
    {
      OUTA = OUTA ^ (1<<26);
    }    
    BufferFull[bufferToSend] = FALSE;    
    bufferToSend = (bufferToSend+1) % BufferCount;
  }
}
 